from ._synergy_file_reader import SynergyFile, SynergyPlate, SynergyResult

try:
	from .version import version as __version__
except ImportError:
	from warnings import warn
	warn('Failed to find (autogenerated) version.py. Do not worry about this unless you really need to know the version.')
